<?php
if (!defined('ABSPATH')) exit;

class WSKO_Controller_Social extends WSKO_Controller
{
	//Options
	public $uses_timespan = true;
	public $uses_fixed_timespan = true;
	public $icon = "share-alt";
	public $link = "social";
	
	public $ajax_actions = array('table_social');

	public $template_folder = "social";
	
	public $subpages = array(
		'facebook' => array('title' => 'Facebook', 'template' => 'social/page-facebook.php'),
		'twitter' => array('title' => 'Twitter', 'template' => 'social/page-twitter.php'),
	);
	
	public function get_title()
	{
		return __('Social', 'wsko');
	}
	
	public function get_subpage_title($subpage)
	{
		switch($subpage)
		{
			case 'facebook': return __('Facebook', 'wsko');
			case 'twitter': return __('Twitter', 'wsko');
		}
		return '';
	}

	public function get_knowledge_base_tags($subpage)
	{
		return array("social");
	}
	
	public function load_lazy_page_data($lazy_data)
	{
		if (!$this->can_execute_action(false))
			return false;
		
		$notif = "";
		$data = array();
		
		$page = $this->get_current_subpage();
		if ($page)
		{
			switch ($page)
			{
				case 'facebook':
				if (WSKO_Class_Social::get_social_logged_in('facebook'))
				{
					$facebook_data = $this->get_cached_var('facebook_data');
					if ($facebook_data)
					{
						$data['total_likes'] = $facebook_data['yesterday']['follower'];
						$data['total_reach'] = $facebook_data['yesterday']['reach'];
						$data['total_views'] = $facebook_data['yesterday']['views'];
						$data['total_engagement'] = $facebook_data['yesterday']['engagement'];
						
						if ($facebook_data['history_follower'])
							$data['chart_history_follower'] = WSKO_Class_Template::render_chart('area', array('Date', 'Follower'), $facebook_data['history_follower'], array(), true);
						if ($facebook_data['history_follower'])
							$data['chart_history_reach'] = WSKO_Class_Template::render_chart('area', array('Date', 'Reach'), $facebook_data['history_reach'], array(), true);
						if ($facebook_data['history_follower'])
							$data['chart_history_views'] = WSKO_Class_Template::render_chart('area', array('Date', 'Views'), $facebook_data['history_views'], array(), true);
						if ($facebook_data['history_follower'])
							$data['chart_history_engagement'] = WSKO_Class_Template::render_chart('area', array('Date', 'Engagement'), $facebook_data['history_engagement'], array(), true);
						
						$current_posts = WSKO_Class_Social::get_current_facebook_posts();
						$total_posts = 0;
						$avg_post_likes = 0;
						$avg_post_engagement = 0;
						$avg_post_reach = 0;
						if ($current_posts)
						{
							foreach($current_posts as $p)
							{
								$total_posts++;
								$avg_post_likes += $p->follower;
								$avg_post_engagement += $p->engagement;
								$avg_post_reach += $p->reach;
							}
						}
						$data['total_posts'] = $total_posts;
						if ($total_posts)
						{
							$data['avg_post_likes'] = $avg_post_likes ? round($avg_post_likes/$total_posts, 2) : 0;
							$data['avg_post_engagement'] = $avg_post_engagement ? round($avg_post_engagement/$total_posts, 2) : 0;
							$data['avg_post_reach'] = $avg_post_reach ? round($avg_post_reach/$total_posts, 2) : 0;
						}
						else
						{
							$data['avg_post_likes'] =  "-";
							$data['avg_post_engagement'] = "-";
							$data['avg_post_reach'] = "-";
						}
					}
					else
					{
						$data['total_likes'] = "-";
						$data['total_reach'] = "-";
						$data['total_views'] = "-";
						$data['total_engagement'] = "-";
						
						$data['total_posts'] = "-";
						$data['avg_post_likes'] =  "-";
						$data['avg_post_engagement'] = "-";
						$data['avg_post_reach'] = "-";
					}
				}
				break;
				case 'twitter':
				if (!WSKO_Class_Social::check_twitter_access())
				{
					$twitter_data = $this->get_cached_var('twitter_data');
					if ($twitter_data)
					{
						$data['total_follower'] = $twitter_data['yesterday']['follower'];
						
						if ($twitter_data['history_follower'])
							$data['chart_history_follower'] = WSKO_Class_Template::render_chart('area', array('Date', 'Follower'), $twitter_data['history_follower'], array(), true);
						
						$current_posts = WSKO_Class_Social::get_current_twitter_posts();
						$total_posts = 0;
						$total_post_likes = 0;
						$avg_post_engagement = 0;
						foreach($current_posts as $p)
						{
							$total_posts++;
							$total_post_likes += $p->follower;
							$avg_post_engagement += $p->engagement;
							
						}
						$data['total_post_likes'] = $total_posts;
						if ($total_posts)
						{
							$data['total_post_engagement'] = $avg_post_engagement;
							$data['total_post_likes'] = $total_post_likes;
							$data['avg_post_likes'] = $total_post_likes ? round($total_post_likes/$total_posts, 2) : 0;
							$data['avg_post_engagement'] = $avg_post_engagement ? round($avg_post_engagement/$total_posts, 2) : 0;
						}
						else
						{
							$data['total_post_engagement'] = '-';
							$data['total_post_likes'] = '-';
							$data['avg_post_likes'] = '-';
							$data['avg_post_engagement'] = '-';
						}
					}
					else
					{
						$data['total_follower'] = "-";
						$data['total_posts'] = "-";
						
						$data['total_post_likes'] =  "-";
						$data['total_post_engagement'] = "-";
						$data['avg_post_likes'] =  "-";
						$data['avg_post_engagement'] = "-";
					}
				}
				break;
			}
			return array(
					'success' => true,
					'data' => $data,
					'notif' => $notif
				);
		}
	}
	
	public function action_table_social()
	{
		if (!$this->can_execute_action(false))
			return false;
		
		$data = array();
		$is_custom = false;
		$is_formated = false;
		$arg = isset($_POST['arg']) ? sanitize_text_field($_POST['arg']) : false;
		switch ($arg)
		{
			case '3':
				$current_posts = WSKO_Class_Social::get_current_facebook_posts();
				$data = $current_posts;
				$params = array(
					'specific_keys' => array('data', 'follower', 'engagement', 'reach', 'created', 'data'),
					'format' => array(
						'4' => 'datetime',
						'0' => function($v){ $v = WSKO_Class_Helper::safe_json_decode($v, true); return $v ? ($v['img'] ? '<img src="'.$v['img'].'"/>' : '').(isset($v['content'])?$v['content']:'') : ''; },
						'5' => function($v){ $v = WSKO_Class_Helper::safe_json_decode($v, true); $link = WSKO_Class_Social::get_fb_post_link($v['id']); if (!$link) return ""; return '<a href="'.$link.'" target="_blank"><i class="fa fa-eye"></i></a>'; }
					)
				);
			break;
			case '4':
				$current_posts = WSKO_Class_Social::get_current_twitter_posts();
				$data = $current_posts;
				$params = array(
					'specific_keys' => array('data', 'follower', 'engagement', 'created', 'data'),
					'format' => array(
						'3' => 'datetime',
						'0' => function($v){ $v = WSKO_Class_Helper::safe_json_decode($v, true); return $v ? (isset($v['content'])?$v['content']:'').($v['urls'] ? '(Urls: '.implode(',',$v['urls']).')' : '') : ''; },
						'4' => function($v){ $v = WSKO_Class_Helper::safe_json_decode($v, true); $link = WSKO_Class_Social::get_tw_post_link($v['id']); if (!$link) return ""; return '<a href="'.$link.'" target="_blank"><i class="fa fa-eye"></i></a>'; }
					)
				);
			break;
		}
		if ($data && !$is_custom)
		{
			$data = $this->get_table_data($data, $params, $is_formated);
		}
		return array(
				'success' => true,
				'data' => $data ? $data['data'] : array(),
				'recordsFiltered' => $data ? $data['filtered'] : 0,
				'recordsTotal' => $data ? $data['total'] : 0,
			);
	}
	
	public function get_cached_facebook_data($args)
	{
		$res = array();
		$fb_data = WSKO_Class_Social::get_facebook_data();
		
		if ($fb_data)
			$res['yesterday'] = array('follower' => $fb_data->follower, 'reach' => $fb_data->reach, 'views' => $fb_data->views, 'engagement' => $fb_data->engagement);
		$facebook_history = WSKO_Class_Social::get_facebook_history($this->timespan_start, $this->timespan_end);
		if ($facebook_history)
		{
			$res['history_follower'] = array();
			$res['history_reach'] = array();
			$res['history_views'] = array();
			$res['history_engagement'] = array();
			foreach ($facebook_history as $h)
			{
				$res['history_follower'][] = array($h->time, $h->follower);
				$res['history_reach'][] = array($h->time, $h->reach);
				$res['history_views'][] = array($h->time, $h->views);
				$res['history_engagement'][] = array($h->time, $h->engagement);
			}
		}		
		return $res;
	}
	
	public function get_cached_twitter_data($args)
	{
		$res = array();
		$tw_data = WSKO_Class_Social::get_twitter_data();
		if ($tw_data)
			$res['yesterday'] = array('follower' => $tw_data->follower, 'reach' => $tw_data->reach, 'views' => $tw_data->views, 'engagement' => $tw_data->engagement);
		$twitter_history = WSKO_Class_Social::get_twitter_history($this->timespan_start, $this->timespan_end);
		if ($twitter_history)
		{
			$res['history_follower'] = array();
			$res['history_reach'] = array();
			$res['history_views'] = array();
			$res['history_engagement'] = array();
			foreach ($twitter_history as $h)
			{
				$res['history_follower'][] = array($h->time, $h->follower);
				$res['history_reach'][] = array($h->time, $h->reach);
				$res['history_views'][] = array($h->time, $h->views);
				$res['history_engagement'][] = array($h->time, $h->engagement);
			}
		}
		return $res;
	}
	
	public function get_cached_graph_data($args)
	{
		$res = array('facebook_history' => array(), 'twitter_history' => array());
		$facebook_history = WSKO_Class_Social::get_facebook_history($this->timespan_start, $this->timespan_end);
		foreach ($facebook_history as $h)
		{
			$res['facebook_history'][] = array($h->time, $h->follower);
		}
		
		$twitter_history = WSKO_Class_Social::get_twitter_history($this->timespan_start, $this->timespan_end);
		foreach ($twitter_history as $h)
		{
			$res['twitter_history'][] = array($h->time, $h->follower);
		}
		return $res;
	}
	
	//Singleton
	static $instance;
}
//WSKO_Controller_Social::init_controller(); //temporarily deactivated
?>