<?php

if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class WSKO_Class_Core
{
    public static function install()
    {
        //WSKO_Class_Crons::register_cronjobs();
        WSKO_Class_Cache::check_database();
    }
    
    public static function update()
    {
        WSKO_Class_Cache::check_database();
    }
    
    public static function deinstall()
    {
        WSKO_Class_Crons::deregister_cronjobs();
    }
    
    public static function setup()
    {
        WSKO_Class_Cache::check_database();
        WSKO_Class_Core::set_default_settings();
        WSKO_Class_Crons::deregister_cronjobs();
        WSKO_Class_Crons::register_cronjobs();
    }
    
    public static function set_default_settings()
    {
        //if (WSKO_Class_Core::get_setting('defaults_loaded'))
        //return;
        WSKO_Class_Onpage::set_sitemap_params( array(
            'types' => array(
            'post' => array(
            'freq' => 'monthly',
        ),
            'page' => array(
            'freq' => 'monthly',
        ),
        ),
            'stati' => array( 'publish' ),
        ) );
        if ( !WSKO_Class_Core::has_setting( 'auto_canonical' ) ) {
            WSKO_Class_Core::save_setting( 'auto_canonical', true );
        }
        //WSKO_Class_Core::save_setting('auto_social_snippet', true);
        if ( !WSKO_Class_Core::has_setting( 'auto_social_thumbnail' ) ) {
            WSKO_Class_Core::save_setting( 'auto_social_thumbnail', true );
        }
        if ( !WSKO_Class_Core::has_setting( 'activate_content_optimizer' ) ) {
            WSKO_Class_Core::save_setting( 'activate_content_optimizer', true );
        }
        if ( !WSKO_Class_Core::has_setting( 'auto_post_slug_redirects' ) ) {
            WSKO_Class_Core::save_setting( 'auto_post_slug_redirects', true );
        }
        if ( !WSKO_Class_Core::has_setting( 'automatic_sitemap_ping' ) ) {
            WSKO_Class_Core::save_setting( 'automatic_sitemap_ping', true );
        }
        $post_types = WSKO_Class_Helper::get_public_post_types( 'names' );
        foreach ( $post_types as $pt ) {
            $res_o = WSKO_Class_Onpage::get_meta_object( $pt, 'post_type' );
            
            if ( !$res_o ) {
                $res = array(
                    'title' => '%post:post_title% | %site:blog_name%',
                );
                WSKO_Class_Onpage::set_meta_object( $pt, $res, 'post_type' );
            }
        
        }
        //WSKO_Class_Core::save_setting('defaults_loaded', true);
    }
    
    public static function init_classes( $force = false )
    {
        global  $wsko_initated, $wsko_plugin_path ;
        require_once $wsko_plugin_path . 'includes/internals.php';
        require_once $wsko_plugin_path . 'const.php';
        require_once $wsko_plugin_path . 'classes/class.helper.php';
        if ( $wsko_initated ) {
            return;
        }
        $wsko_initated = true;
        //if (WSKO_Class_Helper::is_wsko_ajax() || WSKO_Class_Helper::is_wsko_page() || WSKO_Class_Helper::is_wsko_cron())
        //WSKO_Class_Helper::activate_cloud_logging();
        require_once $wsko_plugin_path . 'classes/class.cache.php';
        require_once $wsko_plugin_path . 'classes/class.onpage.php';
        require_once $wsko_plugin_path . 'classes/class.social.php';
        require_once $wsko_plugin_path . 'classes/class.search.php';
        require_once $wsko_plugin_path . 'classes/class.template.php';
        require_once $wsko_plugin_path . 'classes/class.crons.php';
        require_once $wsko_plugin_path . 'classes/class.import.php';
        require_once $wsko_plugin_path . 'classes/class.knowledge.php';
        WSKO_Class_Core::check_version();
        require_once $wsko_plugin_path . 'admin/controller/controller.php';
        
        if ( WSKO_Class_Core::is_configured() ) {
            require_once $wsko_plugin_path . 'admin/controller/controller.dashboard.php';
            require_once $wsko_plugin_path . 'admin/controller/controller.setup.php';
        } else {
            require_once $wsko_plugin_path . 'admin/controller/controller.setup.php';
            require_once $wsko_plugin_path . 'admin/controller/controller.dashboard.php';
        }
        
        require_once $wsko_plugin_path . 'admin/controller/controller.search.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.onpage.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.tools.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.social.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.settings.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.knowledge.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.account.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.optimizer.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.download.php';
        require_once $wsko_plugin_path . 'admin/controller/controller.info.php';
        //init necessaries
        
        if ( WSKO_Class_Core::is_configured() ) {
            WSKO_Class_Crons::add_cronjobs();
            WSKO_Class_Onpage::register_hooks();
            WSKO_Class_Search::register_hooks();
        }
        
        
        if ( is_admin() ) {
            //if (!defined('DOING_AJAX') || !DOING_AJAX)
            //{
            require_once $wsko_plugin_path . 'admin/admin.php';
            //}
            //Load Admin
            add_action( 'plugins_loaded', function () {
                global  $wsko_plugin_path ;
                $inst = new WSKO_Class_Core();
                add_filter( 'plugin_locale', array( $inst, 'modify_plugin_locale' ) );
                load_plugin_textdomain( 'wsko', false, basename( dirname( dirname( __FILE__ ) ) ) . '/languages/' );
                remove_filter( 'plugin_locale', array( $inst, 'modify_plugin_locale' ) );
                WSKO_AdminMenu::get_instance();
            } );
            if ( WSKO_Class_Core::get_option( 'refresh_permalinks' ) ) {
                add_action( 'wp_loaded', function () {
                    flush_rewrite_rules();
                } );
            }
            add_action( 'init', function () {
                WSKO_Class_Helper::add_error_post_type();
                WSKO_Class_Knowledge::add_post_types();
            } );
            add_action( 'admin_init', function () {
            } );
            add_action( 'shutdown', function () {
                WSKO_Class_Cache::session_cache_ntfs_unlock();
            } );
        }
        
        //automatic fast reoccurring cronjobs
        
        if ( WSKO_Class_Core::is_configured() ) {
            if ( WSKO_Class_Search::get_se_token() && WSKO_Class_Search::get_ga_client_se() && !WSKO_Class_Core::get_option( 'search_query_first_run' ) && !WSKO_Class_Core::get_option( 'search_query_running' ) && wp_next_scheduled( 'wsko_cache_keywords' ) > time() + 60 ) {
                WSKO_Class_Crons::bind_keyword_update( true );
            }
            if ( WSKO_Class_Core::get_option( 'onpage_analysis_running' ) && WSKO_Class_Core::get_option( 'last_onpage_segment_fetch' ) < time() - (WSKO_LRS_TIMEOUT + 10) && wp_next_scheduled( 'wsko_onpage_analysis' ) > time() + 60 ) {
                WSKO_Class_Crons::bind_onpage_analysis( true );
            }
        }
    
    }
    
    public function modify_plugin_locale( $locale )
    {
        $lang = WSKO_Class_Core::get_setting( 'plugin_lang' );
        
        if ( !$lang || $lang === 'auto' ) {
            return $locale;
        } else {
            return $lang;
        }
        
        //return $locale;
    }
    
    public static function check_version( $force = false )
    {
        $version_pre = get_option( 'wsko_init' );
        
        if ( $version_pre && is_array( $version_pre ) && isset( $version_pre['version'] ) ) {
            $version_pre = $version_pre['version'];
        } else {
            $version_pre = false;
        }
        
        $wsko_data = WSKO_Class_Core::get_data();
        
        if ( $force || $version_pre && version_compare( $version_pre, WSKO_VERSION, '<' ) ) {
            $update_needed = true;
            
            if ( $version_pre ) {
                $update_needed = false;
                if ( version_compare( $version_pre, '2.0', '<' ) ) {
                    //Reinit below 2.0
                    $update_needed = true;
                }
                
                if ( version_compare( $version_pre, '2.0.4', '<' ) ) {
                    delete_option( 'wsko_cache_snapshot' );
                    wp_clear_scheduled_hook( 'wsko_cache_snapshot' );
                    //remove old cache snapshot
                    if ( WSKO_Class_Core::is_configured() ) {
                        WSKO_Class_Crons::bind_daily_maintenance();
                    }
                    WSKO_Class_Core::save_setting( 'auto_social_snippet', true );
                    WSKO_Class_Core::save_setting( 'auto_social_thumbnail', true );
                    $wsko_data = WSKO_Class_Core::get_data( true );
                    //update fields for further updates
                }
                
                
                if ( version_compare( $version_pre, '2.0.5', '<' ) ) {
                    wp_clear_scheduled_hook( 'wsko_cache_expire' );
                    $onpage_data = WSKO_Class_Onpage::get_onpage_data();
                    
                    if ( isset( $onpage_data['priority_kewords'] ) ) {
                        $onpage_data['priority_keywords'] = $onpage_data['priority_kewords'];
                        unset( $onpage_data['priority_kewords'] );
                        WSKO_Class_Onpage::set_onpage_data( $onpage_data );
                    }
                    
                    wp_clear_scheduled_hook( 'wsko_check_timeout' );
                    WSKO_Class_Core::save_setting( 'auto_post_slug_redirects', true );
                    $wsko_data = WSKO_Class_Core::get_data( true );
                    //update fields for further updates
                }
                
                
                if ( version_compare( $version_pre, '2.0.6', '<' ) ) {
                    $gen_params = WSKO_Class_Onpage::get_sitemap_params();
                    if ( isset( $gen_params['excluded_posts'] ) ) {
                        unset( $gen_params['excluded_posts'] );
                    }
                    WSKO_Class_Onpage::set_sitemap_params( $gen_params );
                    $wsko_data = WSKO_Class_Core::get_data( true );
                    //update fields for further updates
                }
                
                if ( version_compare( $version_pre, '2.0.8', '<' ) ) {
                    
                    if ( !isset( $wsko_data['install_time'] ) ) {
                        $wsko_data['install_time'] = time();
                        WSKO_Class_Core::save_data( $wsko_data );
                        $wsko_data = WSKO_Class_Core::get_data( true );
                        //update fields for further updates
                    }
                
                }
                
                if ( version_compare( $version_pre, '2.1', '<' ) ) {
                    wsko_fs()->connect_again();
                    WSKO_Class_Crons::unbind_social_cache();
                    $onpage_data = WSKO_Class_Onpage::get_onpage_data();
                    $has_u = false;
                    
                    if ( isset( $onpage_data['priority_keywords'] ) && $onpage_data['priority_keywords'] ) {
                        foreach ( $onpage_data['priority_keywords'] as $k => $pks ) {
                            foreach ( $pks as $k2 => $pk ) {
                                if ( is_scalar( $pk ) ) {
                                    $onpage_data['priority_keywords'][$k][$k2] = array(
                                        'prio'    => $pk,
                                        'similar' => array(),
                                    );
                                }
                            }
                        }
                        foreach ( $onpage_data['priority_keywords'] as $post_id => $pks ) {
                            $data = WSKO_Class_Core::get_post_data( $post_id );
                            $data['priority_keywords'] = $pks;
                            WSKO_Class_Core::set_post_data( $post_id, $data );
                        }
                        unset( $onpage_data['priority_keywords'] );
                        $has_u = true;
                    }
                    
                    
                    if ( isset( $onpage_data['post_metas'] ) && $onpage_data['post_metas'] ) {
                        foreach ( $onpage_data['post_metas'] as $post_id => $meta ) {
                            WSKO_Class_Onpage::set_meta_object( $post_id, $meta, 'post_id' );
                        }
                        unset( $onpage_data['post_metas'] );
                        $has_u = true;
                    }
                    
                    
                    if ( isset( $onpage_data['post_term_metas'] ) && $onpage_data['post_term_metas'] ) {
                        foreach ( $onpage_data['post_term_metas'] as $tax_term_id => $meta ) {
                            WSKO_Class_Onpage::set_meta_object( $tax_term_id, $meta, 'post_term' );
                        }
                        unset( $onpage_data['post_term_metas'] );
                        $has_u = true;
                    }
                    
                    //case 'post_term': $from_m = ''; break;
                    
                    if ( isset( $onpage_data['global_analysis'] ) ) {
                        unset( $onpage_data['global_analysis'] );
                        if ( wp_next_scheduled( 'wsko_onpage_analysis' ) ) {
                            //rebind if set
                            WSKO_Class_Crons::bind_onpage_analysis( true );
                        }
                        $has_u = true;
                    }
                    
                    if ( $has_u ) {
                        WSKO_Class_Onpage::set_onpage_data( $onpage_data );
                    }
                    WSKO_Class_Cache::clear_session_cache( false );
                    WSKO_Class_Cache::delete_cache();
                    WSKO_Class_Cache::check_database();
                    WSKO_Class_Core::save_option( 'search_query_first_run', false );
                    if ( WSKO_Class_Core::is_configured() ) {
                        WSKO_Class_Crons::bind_keyword_update( true );
                    }
                    if ( WSKO_Class_Core::get_setting( 'use_leightweight_cache' ) ) {
                        WSKO_Class_Core::save_setting( 'use_lightweight_cache', true );
                    }
                    if ( $has_u ) {
                        $wsko_data = WSKO_Class_Core::get_data( true );
                    }
                    //update fields for further updates
                }
            
            }
            
            WSKO_Class_Cache::update_database();
            
            if ( $update_needed ) {
                $wsko_data = WSKO_Class_Core::get_data_default();
            } else {
                $wsko_data['version'] = WSKO_VERSION;
            }
            
            if ( version_compare( $version_pre, WSKO_VERSION, '<' ) ) {
                $wsko_data['version_pre'] = $version_pre;
            }
            WSKO_Class_Core::save_data( $wsko_data );
        }
    
    }
    
    public static function is_configured( $spec = false )
    {
        $wsko_data = WSKO_Class_Core::get_data();
        
        if ( $spec ) {
            
            if ( $spec === 'core' ) {
                return isset( $wsko_data['configured'] ) && $wsko_data['configured'];
            } else {
                if ( $spec === 'premium' ) {
                    return isset( $wsko_data['configured_prem'] ) && $wsko_data['configured_prem'];
                }
            }
        
        } else {
            $res = isset( $wsko_data['configured'] ) && $wsko_data['configured'];
            if ( $res ) {
            }
            return $res;
        }
    
    }
    
    public static function set_configured( $to = true )
    {
        
        if ( current_user_can( 'manage_options' ) ) {
            $wsko_data = WSKO_Class_Core::get_data();
            
            if ( $to ) {
                $wsko_data['configured'] = true;
            } else {
                unset( $wsko_data['configured'] );
            }
            
            WSKO_Class_Core::save_data( $wsko_data );
            return true;
        }
        
        return false;
    }
    
    public static function include_lib( $lib )
    {
        global  $wsko_plugin_path ;
        switch ( $lib ) {
            case 'forceutf8':
                if ( !class_exists( '\\ForceUTF8\\Encoding' ) ) {
                    require_once $wsko_plugin_path . 'includes/forceutf8/src/ForceUTF8/Encoding.php';
                }
                break;
            case 'iso_lang':
                if ( !class_exists( '\\League\\ISO3166\\ISO3166' ) ) {
                    require_once $wsko_plugin_path . 'includes/iso3166/vendor/autoload.php';
                }
                break;
            case 'google':
                if ( !defined( 'WSKO_GOOGLE_INCLUDE_FAILED' ) ) {
                    
                    if ( !class_exists( 'Google_Client' ) ) {
                        //require_once($wsko_plugin_path . 'includes/google-api-php-client-2.1.0/vendor/autoload.php');
                        //require_once($wsko_plugin_path . 'includes/google-api-php-client-2.2.1/vendor/autoload.php');
                        require_once $wsko_plugin_path . 'includes/google-api-php-client-2.2.2/vendor/autoload.php';
                        
                        if ( class_exists( 'Google_Client' ) ) {
                            define( 'WSKO_GOOGLE_INCLUDE_FAILED', false );
                            WSKO_Class_Search::check_token();
                        } else {
                            define( 'WSKO_GOOGLE_INCLUDE_FAILED', true );
                        }
                    
                    } else {
                        define( 'WSKO_GOOGLE_INCLUDE_FAILED', true );
                    }
                
                }
                break;
            case 'facebook':
                if ( !defined( 'WSKO_FACEBOOK_INCLUDE_FAILED' ) ) {
                    
                    if ( !class_exists( '\\Facebook\\Facebook' ) ) {
                        require_once $wsko_plugin_path . 'includes/php-graph-sdk-5.x/src/Facebook/autoload.php';
                        
                        if ( class_exists( '\\Facebook\\Facebook' ) ) {
                            define( 'WSKO_FACEBOOK_INCLUDE_FAILED', false );
                        } else {
                            define( 'WSKO_FACEBOOK_INCLUDE_FAILED', true );
                        }
                    
                    } else {
                        define( 'WSKO_FACEBOOK_INCLUDE_FAILED', true );
                    }
                
                }
                break;
            case 'twitter':
                if ( !defined( 'WSKO_TWITTER_INCLUDE_FAILED' ) ) {
                    
                    if ( !class_exists( 'TwitterAPIExchange' ) ) {
                        require_once $wsko_plugin_path . 'includes/twitter-exchange-api/TwitterAPIExchange.php';
                        
                        if ( class_exists( 'TwitterAPIExchange' ) ) {
                            define( 'WSKO_TWITTER_INCLUDE_FAILED', false );
                        } else {
                            define( 'WSKO_TWITTER_INCLUDE_FAILED', true );
                        }
                    
                    } else {
                        define( 'WSKO_TWITTER_INCLUDE_FAILED', true );
                    }
                
                }
                break;
            case 'freemius':
                if ( !function_exists( 'fs_dynamic_init' ) ) {
                    require_once $wsko_plugin_path . '/includes/freemius/start.php';
                }
                break;
        }
    }
    
    public static function get_current_controller()
    {
        $inst = WSKO_AdminMenu::get_instance();
        if ( $inst ) {
            return $contr = $inst->controller;
        }
        return false;
    }
    
    public static function get_critical_errors()
    {
        $res = array();
        global  $wp_version ;
        if ( version_compare( $wp_version, '4.4', '<' ) ) {
            $res['incompatible_wp'] = true;
        }
        if ( version_compare( PHP_VERSION, '5.5', '<' ) ) {
            $res['incompatible_php'] = true;
        }
        
        if ( !file_exists( WP_CONTENT_DIR . '/bst/' ) && !is_writable( WP_CONTENT_DIR ) ) {
            $res['content_dir_access'] = true;
        } else {
            if ( !file_exists( WP_CONTENT_DIR . '/bst/' ) ) {
                mkdir( WP_CONTENT_DIR . '/bst/' );
            }
            if ( !is_writable( WP_CONTENT_DIR . '/bst/' ) || !is_readable( WP_CONTENT_DIR . '/bst/' ) ) {
                $res['bst_dir_access'] = true;
            }
        }
        
        return ( empty($res) ? false : $res );
    }
    
    public static function get_data( $refetch = false )
    {
        return WSKO_Class_Cache::get_wp_option( 'wsko_init', function ( $wsko_data ) {
            
            if ( !$wsko_data || !is_array( $wsko_data ) ) {
                $wsko_data_t = WSKO_Class_Core::get_data_default();
                WSKO_Class_Core::save_data( $wsko_data_t );
                $wsko_data = $wsko_data_t;
            }
            
            return $wsko_data;
        }, $refetch );
    }
    
    public static function save_data( $wsko_data )
    {
        WSKO_Class_Cache::save_wp_option( 'wsko_init', $wsko_data );
    }
    
    public static function has_setting( $key )
    {
        $wsko_data = WSKO_Class_Core::get_data();
        if ( isset( $wsko_data['settings'] ) && is_array( $wsko_data['settings'] ) && isset( $wsko_data['settings'][$key] ) ) {
            return true;
        }
        return false;
    }
    
    public static function get_setting( $key )
    {
        $wsko_data = WSKO_Class_Core::get_data();
        if ( isset( $wsko_data['settings'] ) && is_array( $wsko_data['settings'] ) && isset( $wsko_data['settings'][$key] ) && $wsko_data['settings'][$key] ) {
            return $wsko_data['settings'][$key];
        }
    }
    
    public static function save_setting( $key, $val )
    {
        
        if ( $key ) {
            $wsko_data = WSKO_Class_Core::get_data();
            
            if ( isset( $wsko_data['settings'] ) && is_array( $wsko_data['settings'] ) ) {
                $wsko_data['settings'][$key] = $val;
            } else {
                $wsko_data['settings'] = array(
                    $key => $val,
                );
            }
            
            WSKO_Class_Core::save_data( $wsko_data );
        }
    
    }
    
    public static function get_user_setting( $key, $user = false )
    {
        $wsko_data = WSKO_Class_Core::get_data();
        if ( $user === false ) {
            $user = get_current_user_id();
        }
        if ( isset( $wsko_data['user_settings'] ) && is_array( $wsko_data['user_settings'] ) && isset( $wsko_data['user_settings'][$user][$key] ) && $wsko_data['user_settings'][$user][$key] ) {
            return $wsko_data['user_settings'][$user][$key];
        }
    }
    
    public static function save_user_setting( $key, $val, $user = false )
    {
        
        if ( $key ) {
            if ( $user === false ) {
                $user = get_current_user_id();
            }
            $wsko_data = WSKO_Class_Core::get_data();
            
            if ( isset( $wsko_data['user_settings'][$user] ) && is_array( $wsko_data['user_settings'][$user] ) ) {
                $wsko_data['user_settings'][$user][$key] = $val;
            } else {
                
                if ( !isset( $wsko_data['user_settings'] ) ) {
                    $wsko_data['user_settings'] = array(
                        $user => array(
                        $key => $val,
                    ),
                    );
                } else {
                    $wsko_data['user_settings'][$user] = array(
                        $key => $val,
                    );
                }
            
            }
            
            WSKO_Class_Core::save_data( $wsko_data );
        }
    
    }
    
    public static function get_options( $refetch = false )
    {
        return WSKO_Class_Cache::get_wp_option( 'wsko_options', false, $refetch );
    }
    
    public static function save_options( $wsko_data )
    {
        WSKO_Class_Cache::save_wp_option( 'wsko_options', $wsko_data );
    }
    
    public static function get_option( $key, $refetch = false )
    {
        $data_options = WSKO_Class_Core::get_options( $refetch );
        if ( $data_options && is_array( $data_options ) && isset( $data_options[$key] ) && $data_options[$key] ) {
            return $data_options[$key];
        }
        return false;
    }
    
    public static function save_option( $key, $val, $refetch = false )
    {
        
        if ( $key ) {
            $data_options = WSKO_Class_Core::get_options( $refetch );
            
            if ( isset( $data_options ) && is_array( $data_options ) ) {
                $data_options[$key] = $val;
            } else {
                $data_options = array(
                    $key => $val,
                );
            }
            
            WSKO_Class_Core::save_options( $data_options );
        }
    
    }
    
    public static function get_data_default()
    {
        return array(
            'version'      => WSKO_VERSION,
            'install_time' => time(),
        );
    }
    
    public static function backup_configuration( $auto = false )
    {
        $backups = get_option( 'wsko_backups' );
        if ( !$backups ) {
            $backups = array();
        }
        
        if ( $auto ) {
            $time = WSKO_Class_Helper::get_midnight();
        } else {
            $time = time();
        }
        
        $backup_data_r = get_option( 'wsko_init' );
        
        if ( $backup_data_r && is_array( $backup_data_r ) && isset( $backup_data_r['version'] ) ) {
            //dont save these
            if ( isset( $backup_data_r['search']['se_token'] ) ) {
                unset( $backup_data_r['search']['se_token'] );
            }
            if ( isset( $backup_data_r['search']['an_token'] ) ) {
                unset( $backup_data_r['search']['an_token'] );
            }
            if ( isset( $backup_data_r['search']['an_profile'] ) ) {
                unset( $backup_data_r['search']['an_profile'] );
            }
            if ( isset( $backup_data_r['knowledge_base_categories'] ) ) {
                unset( $backup_data_r['knowledge_base_categories'] );
            }
            if ( isset( $backup_data_r['q_and_a_categories'] ) ) {
                unset( $backup_data_r['q_and_a_categories'] );
            }
            if ( isset( $backup_data_r['timeout_check'] ) ) {
                unset( $backup_data_r['timeout_check'] );
            }
            if ( isset( $backup_data_r['version'] ) ) {
                unset( $backup_data_r['version'] );
            }
            if ( isset( $backup_data_r['configured'] ) ) {
                unset( $backup_data_r['configured'] );
            }
            if ( isset( $backup_data_r['configured_prem'] ) ) {
                unset( $backup_data_r['configured_prem'] );
            }
            //--
            $backups[$time] = $backup_data = array(
                'time'    => WSKO_Class_Helper::get_current_time(),
                'auto'    => $auto,
                'data'    => $backup_data_r,
                'version' => WSKO_VERSION,
            );
            update_option( 'wsko_backups', $backups );
        }
    
    }
    
    public static function get_configuration_backup( $key )
    {
        $backups = get_option( 'wsko_backups' );
        if ( isset( $backups[$key] ) ) {
            return $backups[$key];
        }
        return false;
    }
    
    public static function load_configuration_backup( $key )
    {
        $backup = WSKO_Class_Core::get_configuration_backup( $key );
        
        if ( $backup && isset( $backup['version'] ) && version_compare( $backup['version'], '2.1', '>=' ) ) {
            $data = $backup['data'];
            $old_data = get_option( 'wsko_init' );
            if ( $old_data ) {
                $data = WSKO_Class_Helper::merge_array_deep( $data, $old_data );
            }
            //$data = $data + $old_data;
            update_option( 'wsko_init', $data );
        }
    
    }
    
    public static function import_configuration_backup( $data_str, $load = true )
    {
        
        if ( $data_str ) {
            $data = WSKO_Class_Helper::safe_json_decode( $data_str, true );
            
            if ( $data && is_array( $data ) && isset( $data['time'] ) && isset( $data['auto'] ) && isset( $data['data'] ) && isset( $data['key'] ) && isset( $data['version'] ) && version_compare( $data['version'], '2.1', '>=' ) ) {
                $key = $data['key'];
                unset( $data['key'] );
                
                if ( $load ) {
                    $new_data = $data['data'];
                    $old_data = get_option( 'wsko_init' );
                    if ( $old_data ) {
                        $new_data = WSKO_Class_Helper::merge_array_deep( $new_data, $old_data );
                    }
                    //$new_data = $new_data + $old_data;
                    update_option( 'wsko_init', $new_data );
                } else {
                    //add
                    $backups = get_option( 'wsko_backups' );
                    if ( !$backups ) {
                        $backups = array();
                    }
                    $backups[$key] = $data;
                    update_option( 'wsko_backups', $backups );
                }
                
                return true;
            }
        
        }
        
        return false;
    }
    
    public static function delete_configuration_backup( $key )
    {
        $backups = get_option( 'wsko_backups' );
        if ( !$backups ) {
            $backups = array();
        }
        
        if ( isset( $backups[$key] ) ) {
            unset( $backups[$key] );
            update_option( 'wsko_backups', $backups );
        }
    
    }
    
    public static function get_plugin_language( $real = false )
    {
        $lang = WSKO_Class_Core::get_setting( 'plugin_lang' );
        if ( $real ) {
            return $lang;
        }
        
        if ( !$lang || $lang == 'auto' ) {
            $lang = get_user_locale();
            if ( !$lang ) {
                $lang = get_locale();
            }
        }
        
        switch ( $lang ) {
            case 'en_EN':
            case 'de_DE':
                break;
            default:
                $lang = 'en_EN';
                break;
        }
        return $lang;
    }
    
    public static function set_post_data( $post_id, $data )
    {
        update_post_meta( $post_id, '_wsko_post_data', $data );
    }
    
    public static function get_post_data( $post_id )
    {
        $res = get_post_meta( $post_id, '_wsko_post_data', true );
        if ( !is_array( $res ) ) {
            $res = array();
        }
        return $res;
    }
    
    public static function delete_post_data()
    {
        global  $wpdb ;
        $table = $wpdb->prefix . 'postmeta';
        $wpdb->delete( $table, array(
            'meta_key' => '_wsko_post_data',
        ) );
    }
    
    public static function set_term_data( $term_id, $data )
    {
        update_term_meta( $term_id, '_wsko_term_data', $data );
    }
    
    public static function get_term_data( $term_id )
    {
        $res = get_term_meta( $term_id, '_wsko_term_data', true );
        if ( !is_array( $res ) ) {
            $res = array();
        }
        return $res;
    }
    
    public static function delete_term_data()
    {
        global  $wpdb ;
        $table = $wpdb->prefix . 'termmeta';
        $wpdb->delete( $table, array(
            'meta_key' => '_wsko_post_data',
        ) );
    }
    
    public static function add_post_data_history(
        $post_id,
        $key,
        $time,
        $data_v
    )
    {
        $data = WSKO_Class_Core::get_post_data( $post_id );
        if ( !isset( $data['history'] ) ) {
            $data['history'] = array();
        }
        if ( !isset( $data['history'][$key] ) ) {
            $data['history'][$key] = array();
        }
        $data['history'][$key][$time] = $data_v;
        $history_timeout = time() - 60 * 60 * 24 * WSKO_HISTORY_LIMIT;
        foreach ( $data['history'][$key] as $k => $d ) {
            if ( $k < $history_timeout ) {
                unset( $data['history'][$key][$k] );
            }
        }
        ksort( $data['history'][$key] );
        WSKO_Class_Core::set_post_data( $post_id, $data );
    }
    
    public static function get_post_data_history( $post_id, $key = false, $sort = true )
    {
        $data = WSKO_Class_Core::get_post_data( $post_id );
        
        if ( $key ) {
            
            if ( isset( $data['history'][$key] ) ) {
                ksort( $data['history'][$key] );
                return $data['history'][$key];
            }
        
        } else {
            if ( isset( $data['history'] ) ) {
                return $data['history'];
            }
        }
        
        return false;
    }
    
    public static function has_account()
    {
        return wsko_fs()->is_registered();
    }
    
    public static function is_opted_in()
    {
        return wsko_fs()->is_registered() && wsko_fs()->is_tracking_allowed();
    }
    
    public static function is_premium()
    {
        return wsko_fs()->is_premium() && wsko_fs()->can_use_premium_code();
    }

}